/*
 *     Botendo
 *     Copyright (C) 2023  limited_dev
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package de.limited_dev.botendo.commands.slash.music.component

import de.limited_dev.botendo.util.MessageUtil
import de.limited_dev.botendo.util.TimeUtil
import dev.kord.core.behavior.interaction.response.DeferredPublicMessageInteractionResponseBehavior
import dev.kord.core.entity.interaction.GuildChatInputCommandInteraction
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.audio.TrackEndEvent
import dev.schlaubi.lavakord.audio.on
import dev.schlaubi.lavakord.rest.TrackResponse
import dev.schlaubi.lavakord.rest.loadItem

object MusicManager {
    private var musicManagerMap: MutableMap<Long, GuildMusicManager>? = null

    init {
        musicManagerMap = HashMap()
    }

    suspend fun addToQueue(
        interaction: GuildChatInputCommandInteraction,
        response: DeferredPublicMessageInteractionResponseBehavior,
        link: Link,
        search: String
    ) {
        addToQueue(interaction, response, link, search, false)
    }

    suspend fun addToQueue(
        interaction: GuildChatInputCommandInteraction,
        response: DeferredPublicMessageInteractionResponseBehavior,
        link: Link,
        search: String,
        silent: Boolean
    ) {
        val player = link.player
        val item = link.loadItem(search)

        player.on<TrackEndEvent> {

        }

        when (item.loadType) {
            TrackResponse.LoadType.TRACK_LOADED -> {
                player.playTrack(item.track)
                if (!silent)
                    MessageUtil.sendEmbedForInteraction(
                        interaction, response,
                        "Playing track from link",
                        "**${item.track.info.title}**\n*Queue*\nby ${item.track.info.author} :: ${
                            TimeUtil.getTimeFormatedRaw(
                                item.track.info.length
                            )
                        }"
                    )
            }

            TrackResponse.LoadType.PLAYLIST_LOADED -> {
                val l = item.tracks.asReversed()
                for (p in l) {
                    player.playTrack(p)
                }
                if (!silent)
                    MessageUtil.sendEmbedForInteraction(
                        interaction, response,
                        "Playing playlist from link",
                        "**${item.tracks.first().info.title}**\n*${item.playlistInfo.name}*\nby ${item.tracks.first().info.author} :: ${
                            TimeUtil.getTimeFormatedRaw(
                                item.tracks.first().info.length
                            )
                        }"
                    )
            }

            TrackResponse.LoadType.SEARCH_RESULT -> {
                player.playTrack(item.tracks.first())
                if (!silent)
                    MessageUtil.sendEmbedForInteraction(
                        interaction, response,
                        "Playing from query",
                        "**${item.tracks.first().info.title}**\n*Queue*\nby ${item.tracks.first().info.author} :: ${
                            TimeUtil.getTimeFormatedRaw(
                                item.tracks.first().info.length
                            )
                        }"
                    )
            }

            TrackResponse.LoadType.NO_MATCHES -> {
                if (!silent)
                    MessageUtil.sendEmbedForInteraction(interaction, response, "Not found", "There were not matches.")
            }

            TrackResponse.LoadType.LOAD_FAILED -> {
                if (!silent)
                    MessageUtil.sendEmbedForInteraction(
                        interaction,
                        response,
                        "Load failed",
                        "There was an error while loading."
                    )
            }
        }
    }
}
