/*
 *     Botendo
 *     Copyright (C) 2023  limited_dev
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package de.limited_dev.botendo.util

import dev.kord.core.behavior.interaction.response.DeferredPublicMessageInteractionResponseBehavior
import dev.kord.core.behavior.interaction.response.respond
import dev.kord.core.entity.interaction.GuildChatInputCommandInteraction
import dev.kord.rest.builder.message.EmbedBuilder
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

object MessageUtil {
    private val dtf: DateTimeFormatter = DateTimeFormatter.ofPattern("dd/MM/yyyy @ HH:mm:ss")

    suspend fun sendEmbedForInteraction(
        interaction: GuildChatInputCommandInteraction,
        response: DeferredPublicMessageInteractionResponseBehavior,
        title: String,
        description: String
    ) {
        response.respond {
            embeds = mutableListOf(
                getEmbed(
                    title,
                    description,
                    interaction.user.username + "#" + interaction.user.discriminator
                )
            )
        }
    }

    suspend fun sendEmbedForInteractionWithImage(
        interaction: GuildChatInputCommandInteraction,
        response: DeferredPublicMessageInteractionResponseBehavior,
        title: String,
        description: String,
        thumbnailUrl: String
    ) {
        response.respond {
            embeds = mutableListOf(
                getEmbedWithImage(
                    title,
                    description,
                    interaction.user.username + "#" + interaction.user.discriminator,
                    thumbnailUrl
                )
            )
        }
    }

    fun getEmbed(title: String, description: String, source: String): EmbedBuilder {
        val now: LocalDateTime = LocalDateTime.now()
        val ebb = EmbedBuilder()
        ebb.title = title
        ebb.description = description
        ebb.footer = EmbedBuilder.Footer()
        ebb.footer!!.text = ">" + dtf.format(now) + " - $source"
        return ebb
    }

    fun getEmbedWithImage(title: String, description: String, source: String, thumbnailUrl: String): EmbedBuilder {
        val ebb = getEmbed(title, description, source)
        ebb.thumbnail = EmbedBuilder.Thumbnail()
        ebb.thumbnail!!.url = thumbnailUrl
        return ebb
    }
}
